@@ -1,25 +1,32 @@
 # Human friendly input/output in Python.
 #
 # Author: Peter Odding <peter@peterodding.com>
-# Last Change: February 9, 2020
+# Last Change: February 16, 2020
 # URL: https://humanfriendly.readthedocs.io
 
 """
-Interaction with UNIX terminals.
+Interaction with interactive text terminals.
 
-The :mod:`~humanfriendly.terminal` module makes it easy to interact with UNIX
-terminals and format text for rendering on UNIX terminals. If the terms used in
-the documentation of this module don't make sense to you then please refer to
-the `Wikipedia article on ANSI escape sequences`_ for details about how ANSI
-escape sequences work.
+The :mod:`~humanfriendly.terminal` module makes it easy to interact with
+interactive text terminals and format text for rendering on such terminals. If
+the terms used in the documentation of this module don't make sense to you then
+please refer to the `Wikipedia article on ANSI escape sequences`_ for details
+about how ANSI escape sequences work.
+
+This module was originally developed for use on UNIX systems, but since then
+Windows 10 gained native support for ANSI escape sequences and this module was
+enhanced to recognize and support this. For details please refer to the
+:func:`enable_ansi_support()` function.
 
 .. _Wikipedia article on ANSI escape sequences: http://en.wikipedia.org/wiki/ANSI_escape_code#Sequence_elements
 """
 
 # Standard library modules.
 import codecs
+import distutils.spawn
 import numbers
 import os
+import platform
 import re
 import subprocess
 import sys
@@ -38,7 +45,8 @@ except ImportError:
 # Modules included in our package. We import find_meta_variables() here to
 # preserve backwards compatibility with older versions of humanfriendly where
 # that function was defined in this module.
-from humanfriendly.compat import HTMLParser, StringIO, coerce_string, name2codepoint, is_unicode, unichr
+from humanfriendly.compat import HTMLParser, StringIO, coerce_string, name2codepoint, on_windows, is_unicode, unichr
+from humanfriendly.decorators import cached
 from humanfriendly.text import compact_empty_lines, concatenate, format
 from humanfriendly.usage import find_meta_variables, format_usage  # NOQA
 
@@ -63,10 +71,12 @@ __all__ = (
     'auto_encode',
     'clean_terminal_output',
     'connected_to_terminal',
+    'enable_ansi_support',
     'find_terminal_size',
     'find_terminal_size_using_ioctl',
     'find_terminal_size_using_stty',
     'get_pager_command',
+    'have_windows_native_ansi_support',
     'html_to_ansi',
     'message',
     'output',
@@ -132,80 +142,6 @@ message of the ``humanfriendly`` program). If the environment variable
 """
 
 
-def output(text, *args, **kw):
-    """
-    Print a formatted message to the standard output stream.
-
-    For details about argument handling please refer to
-    :func:`~humanfriendly.text.format()`.
-
-    Renders the message using :func:`~humanfriendly.text.format()` and writes
-    the resulting string (followed by a newline) to :data:`sys.stdout` using
-    :func:`auto_encode()`.
-    """
-    auto_encode(sys.stdout, coerce_string(text) + '\n', *args, **kw)
-
-
-def message(text, *args, **kw):
-    """
-    Print a formatted message to the standard error stream.
-
-    For details about argument handling please refer to
-    :func:`~humanfriendly.text.format()`.
-
-    Renders the message using :func:`~humanfriendly.text.format()` and writes
-    the resulting string (followed by a newline) to :data:`sys.stderr` using
-    :func:`auto_encode()`.
-    """
-    auto_encode(sys.stderr, coerce_string(text) + '\n', *args, **kw)
-
-
-def warning(text, *args, **kw):
-    """
-    Show a warning message on the terminal.
-
-    For details about argument handling please refer to
-    :func:`~humanfriendly.text.format()`.
-
-    Renders the message using :func:`~humanfriendly.text.format()` and writes
-    the resulting string (followed by a newline) to :data:`sys.stderr` using
-    :func:`auto_encode()`.
-
-    If :data:`sys.stderr` is connected to a terminal that supports colors,
-    :func:`ansi_wrap()` is used to color the message in a red font (to make
-    the warning stand out from surrounding text).
-    """
-    text = coerce_string(text)
-    if terminal_supports_colors(sys.stderr):
-        text = ansi_wrap(text, color='red')
-    auto_encode(sys.stderr, text + '\n', *args, **kw)
-
-
-def auto_encode(stream, text, *args, **kw):
-    """
-    Reliably write Unicode strings to the terminal.
-
-    :param stream: The file-like object to write to (a value like
-                   :data:`sys.stdout` or :data:`sys.stderr`).
-    :param text: The text to write to the stream (a string).
-    :param args: Refer to :func:`~humanfriendly.text.format()`.
-    :param kw: Refer to :func:`~humanfriendly.text.format()`.
-
-    Renders the text using :func:`~humanfriendly.text.format()` and writes it
-    to the given stream. If an :exc:`~exceptions.UnicodeEncodeError` is
-    encountered in doing so, the text is encoded using :data:`DEFAULT_ENCODING`
-    and the write is retried. The reasoning behind this rather blunt approach
-    is that it's preferable to get output on the command line in the wrong
-    encoding then to have the Python program blow up with a
-    :exc:`~exceptions.UnicodeEncodeError` exception.
-    """
-    text = format(text, *args, **kw)
-    try:
-        stream.write(text)
-    except UnicodeEncodeError:
-        stream.write(codecs.encode(text, DEFAULT_ENCODING))
-
-
 def ansi_strip(text, readline_hints=True):
     """
     Strip ANSI escape sequences from the given string.
@@ -360,26 +296,29 @@ def ansi_wrap(text, **kw):
         return text
 
 
-def readline_wrap(expr):
+def auto_encode(stream, text, *args, **kw):
     """
-    Wrap an ANSI escape sequence in `readline hints`_.
+    Reliably write Unicode strings to the terminal.
 
-    :param text: The text with the escape sequence to wrap (a string).
-    :returns: The wrapped text.
+    :param stream: The file-like object to write to (a value like
+                   :data:`sys.stdout` or :data:`sys.stderr`).
+    :param text: The text to write to the stream (a string).
+    :param args: Refer to :func:`~humanfriendly.text.format()`.
+    :param kw: Refer to :func:`~humanfriendly.text.format()`.
 
-    .. _readline hints: http://superuser.com/a/301355
+    Renders the text using :func:`~humanfriendly.text.format()` and writes it
+    to the given stream. If an :exc:`~exceptions.UnicodeEncodeError` is
+    encountered in doing so, the text is encoded using :data:`DEFAULT_ENCODING`
+    and the write is retried. The reasoning behind this rather blunt approach
+    is that it's preferable to get output on the command line in the wrong
+    encoding then to have the Python program blow up with a
+    :exc:`~exceptions.UnicodeEncodeError` exception.
     """
-    return '\001' + expr + '\002'
-
-
-def readline_strip(expr):
-    """
-    Remove `readline hints`_ from a string.
-
-    :param text: The text to strip (a string).
-    :returns: The stripped text.
-    """
-    return expr.replace('\001', '').replace('\002', '')
+    text = format(text, *args, **kw)
+    try:
+        stream.write(text)
+    except UnicodeEncodeError:
+        stream.write(codecs.encode(text, DEFAULT_ENCODING))
 
 
 def clean_terminal_output(text):
@@ -420,8 +359,8 @@ def clean_terminal_output(text):
       contain only the closing end of the ANSI escape sequence pair. Tracking
       this kind of complexity requires a state machine and proper parsing.
 
-    .. _capturer: https://pypi.python.org/pypi/capturer
-    .. _coloredlogs: https://pypi.python.org/pypi/coloredlogs
+    .. _capturer: https://pypi.org/project/capturer
+    .. _coloredlogs: https://pypi.org/project/coloredlogs
     """
     cleaned_lines = []
     current_line = ''
@@ -474,38 +413,56 @@ def connected_to_terminal(stream=None):
         return False
 
 
-def html_to_ansi(data, callback=None):
+@cached
+def enable_ansi_support():
     """
-    Convert HTML with simple text formatting to text with ANSI escape sequences.
+    Try to enable support for ANSI escape sequences (required on Windows).
 
-    :param data: The HTML to convert (a string).
-    :param callback: Optional callback to pass to :class:`HTMLConverter`.
-    :returns: Text with ANSI escape sequences (a string).
+    :returns: :data:`True` if ANSI is supported, :data:`False` otherwise.
 
-    Please refer to the documentation of the :class:`HTMLConverter` class for
-    details about the conversion process (like which tags are supported) and an
-    example with a screenshot.
+    This functions checks for the following supported configurations, in the
+    given order:
+
+    1. On Windows, if :func:`have_windows_native_ansi_support()` confirms
+       native support for ANSI escape sequences :mod:`ctypes` will be used to
+       enable this support.
+
+    2. On Windows, if the environment variable ``$ANSICON`` is set nothing is
+       done because it is assumed that support for ANSI escape sequences has
+       already been enabled via `ansicon <https://github.com/adoxa/ansicon>`_.
+
+    3. On Windows, an attempt is made to import and initialize the Python
+       package :pypi:`colorama` instead (of course for this to work
+       :pypi:`colorama` has to be installed).
+
+    4. On other platforms this function calls :func:`connected_to_terminal()`
+       to determine whether ANSI escape sequences are supported (that is to
+       say all platforms that are not Windows are assumed to support ANSI
+       escape sequences natively, without weird contortions like above).
+
+       This makes it possible to call :func:`enable_ansi_support()`
+       unconditionally without checking the current platform.
+
+    The :func:`~humanfriendly.decorators.cached` decorator is used to ensure
+    that this function is only executed once, but its return value remains
+    available on later calls.
     """
-    converter = HTMLConverter(callback=callback)
-    return converter(data)
-
-
-def terminal_supports_colors(stream=None):
-    """
-    Check if a stream is connected to a terminal that supports ANSI escape sequences.
-
-    :param stream: The stream to check (a file-like object,
-                   defaults to :data:`sys.stdout`).
-    :returns: :data:`True` if the terminal supports ANSI escape sequences,
-              :data:`False` otherwise.
-
-    This function is inspired by the implementation of
-    `django.core.management.color.supports_color()
-    <https://github.com/django/django/blob/master/django/core/management/color.py>`_.
-    """
-    return (sys.platform != 'Pocket PC' and
-            (sys.platform != 'win32' or 'ANSICON' in os.environ) and
-            connected_to_terminal(stream))
+    if have_windows_native_ansi_support():
+        import ctypes
+        ctypes.windll.kernel32.SetConsoleMode(ctypes.windll.kernel32.GetStdHandle(-11), 7)
+        ctypes.windll.kernel32.SetConsoleMode(ctypes.windll.kernel32.GetStdHandle(-12), 7)
+        return True
+    elif on_windows():
+        if 'ANSICON' in os.environ:
+            return True
+        try:
+            import colorama
+            colorama.init()
+            return True
+        except ImportError:
+            return False
+    else:
+        return connected_to_terminal()
 
 
 def find_terminal_size():
@@ -593,53 +550,6 @@ def find_terminal_size_using_stty():
     return tuple(map(int, tokens))
 
 
-def usage(usage_text):
-    """
-    Print a human friendly usage message to the terminal.
-
-    :param text: The usage message to print (a string).
-
-    This function does two things:
-
-    1. If :data:`sys.stdout` is connected to a terminal (see
-       :func:`connected_to_terminal()`) then the usage message is formatted
-       using :func:`.format_usage()`.
-    2. The usage message is shown using a pager (see :func:`show_pager()`).
-    """
-    if terminal_supports_colors(sys.stdout):
-        usage_text = format_usage(usage_text)
-    show_pager(usage_text)
-
-
-def show_pager(formatted_text, encoding=DEFAULT_ENCODING):
-    """
-    Print a large text to the terminal using a pager.
-
-    :param formatted_text: The text to print to the terminal (a string).
-    :param encoding: The name of the text encoding used to encode the formatted
-                     text if the formatted text is a Unicode string (a string,
-                     defaults to :data:`DEFAULT_ENCODING`).
-
-    When :func:`connected_to_terminal()` returns :data:`True` a pager is used
-    to show the text on the terminal, otherwise the text is printed directly
-    without invoking a pager.
-
-    The use of a pager helps to avoid the wall of text effect where the user
-    has to scroll up to see where the output began (not very user friendly).
-
-    Refer to :func:`get_pager_command()` for details about the command line
-    that's used to invoke the pager.
-    """
-    if connected_to_terminal():
-        command_line = get_pager_command(formatted_text)
-        pager = subprocess.Popen(command_line, stdin=subprocess.PIPE)
-        if is_unicode(formatted_text):
-            formatted_text = formatted_text.encode(encoding)
-        pager.communicate(input=formatted_text)
-    else:
-        output(formatted_text)
-
-
 def get_pager_command(text=None):
     """
     Get the command to show a text on the terminal using a pager.
@@ -680,6 +590,190 @@ def get_pager_command(text=None):
     return command_line
 
 
+@cached
+def have_windows_native_ansi_support():
+    """
+    Check if we're running on a Windows 10 release with native support for ANSI escape sequences.
+
+    :returns: :data:`True` if so, :data:`False` otherwise.
+
+    The :func:`~humanfriendly.decorators.cached` decorator is used as a minor
+    performance optimization. Semantically this should have zero impact because
+    the answer doesn't change in the lifetime of a computer process.
+    """
+    if on_windows():
+        try:
+            # I can't be 100% sure this will never break and I'm not in a
+            # position to test it thoroughly either, so I decided that paying
+            # the price of one additional try / except statement is worth the
+            # additional peace of mind :-).
+            components = tuple(int(c) for c in platform.version().split('.'))
+            return components >= (10, 0, 14393)
+        except Exception:
+            pass
+    return False
+
+
+def html_to_ansi(data, callback=None):
+    """
+    Convert HTML with simple text formatting to text with ANSI escape sequences.
+
+    :param data: The HTML to convert (a string).
+    :param callback: Optional callback to pass to :class:`HTMLConverter`.
+    :returns: Text with ANSI escape sequences (a string).
+
+    Please refer to the documentation of the :class:`HTMLConverter` class for
+    details about the conversion process (like which tags are supported) and an
+    example with a screenshot.
+    """
+    converter = HTMLConverter(callback=callback)
+    return converter(data)
+
+
+def message(text, *args, **kw):
+    """
+    Print a formatted message to the standard error stream.
+
+    For details about argument handling please refer to
+    :func:`~humanfriendly.text.format()`.
+
+    Renders the message using :func:`~humanfriendly.text.format()` and writes
+    the resulting string (followed by a newline) to :data:`sys.stderr` using
+    :func:`auto_encode()`.
+    """
+    auto_encode(sys.stderr, coerce_string(text) + '\n', *args, **kw)
+
+
+def output(text, *args, **kw):
+    """
+    Print a formatted message to the standard output stream.
+
+    For details about argument handling please refer to
+    :func:`~humanfriendly.text.format()`.
+
+    Renders the message using :func:`~humanfriendly.text.format()` and writes
+    the resulting string (followed by a newline) to :data:`sys.stdout` using
+    :func:`auto_encode()`.
+    """
+    auto_encode(sys.stdout, coerce_string(text) + '\n', *args, **kw)
+
+
+def readline_strip(expr):
+    """
+    Remove `readline hints`_ from a string.
+
+    :param text: The text to strip (a string).
+    :returns: The stripped text.
+    """
+    return expr.replace('\001', '').replace('\002', '')
+
+
+def readline_wrap(expr):
+    """
+    Wrap an ANSI escape sequence in `readline hints`_.
+
+    :param text: The text with the escape sequence to wrap (a string).
+    :returns: The wrapped text.
+
+    .. _readline hints: http://superuser.com/a/301355
+    """
+    return '\001' + expr + '\002'
+
+
+def show_pager(formatted_text, encoding=DEFAULT_ENCODING):
+    """
+    Print a large text to the terminal using a pager.
+
+    :param formatted_text: The text to print to the terminal (a string).
+    :param encoding: The name of the text encoding used to encode the formatted
+                     text if the formatted text is a Unicode string (a string,
+                     defaults to :data:`DEFAULT_ENCODING`).
+
+    When :func:`connected_to_terminal()` returns :data:`True` a pager is used
+    to show the text on the terminal, otherwise the text is printed directly
+    without invoking a pager.
+
+    The use of a pager helps to avoid the wall of text effect where the user
+    has to scroll up to see where the output began (not very user friendly).
+
+    Refer to :func:`get_pager_command()` for details about the command line
+    that's used to invoke the pager.
+    """
+    if connected_to_terminal():
+        # Make sure the selected pager command is available.
+        command_line = get_pager_command(formatted_text)
+        if distutils.spawn.find_executable(command_line[0]):
+            pager = subprocess.Popen(command_line, stdin=subprocess.PIPE)
+            if is_unicode(formatted_text):
+                formatted_text = formatted_text.encode(encoding)
+            pager.communicate(input=formatted_text)
+            return
+    output(formatted_text)
+
+
+def terminal_supports_colors(stream=None):
+    """
+    Check if a stream is connected to a terminal that supports ANSI escape sequences.
+
+    :param stream: The stream to check (a file-like object,
+                   defaults to :data:`sys.stdout`).
+    :returns: :data:`True` if the terminal supports ANSI escape sequences,
+              :data:`False` otherwise.
+
+    This function was originally inspired by the implementation of
+    `django.core.management.color.supports_color()
+    <https://github.com/django/django/blob/master/django/core/management/color.py>`_
+    but has since evolved significantly.
+    """
+    if on_windows():
+        # On Windows support for ANSI escape sequences is not a given.
+        have_ansicon = 'ANSICON' in os.environ
+        have_colorama = 'colorama' in sys.modules
+        have_native_support = have_windows_native_ansi_support()
+        if not (have_ansicon or have_colorama or have_native_support):
+            return False
+    return connected_to_terminal(stream)
+
+
+def usage(usage_text):
+    """
+    Print a human friendly usage message to the terminal.
+
+    :param text: The usage message to print (a string).
+
+    This function does two things:
+
+    1. If :data:`sys.stdout` is connected to a terminal (see
+       :func:`connected_to_terminal()`) then the usage message is formatted
+       using :func:`.format_usage()`.
+    2. The usage message is shown using a pager (see :func:`show_pager()`).
+    """
+    if terminal_supports_colors(sys.stdout):
+        usage_text = format_usage(usage_text)
+    show_pager(usage_text)
+
+
+def warning(text, *args, **kw):
+    """
+    Show a warning message on the terminal.
+
+    For details about argument handling please refer to
+    :func:`~humanfriendly.text.format()`.
+
+    Renders the message using :func:`~humanfriendly.text.format()` and writes
+    the resulting string (followed by a newline) to :data:`sys.stderr` using
+    :func:`auto_encode()`.
+
+    If :data:`sys.stderr` is connected to a terminal that supports colors,
+    :func:`ansi_wrap()` is used to color the message in a red font (to make
+    the warning stand out from surrounding text).
+    """
+    text = coerce_string(text)
+    if terminal_supports_colors(sys.stderr):
+        text = ansi_wrap(text, color='red')
+    auto_encode(sys.stderr, text + '\n', *args, **kw)
+
+
 class HTMLConverter(HTMLParser):
 
     """
